# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/03-deployment.ipynb (unless otherwise specified).

__all__ = ['app', 'update_docs']

# Cell
from outage import collation

import typer

# Cell
app = typer.Typer()

# Cell
@app.command()
def update_docs(
    dnos: list=['ukpn', 'ssen', 'wpd', 'sp', 'np', 'enw'],
    postcodes_fp: str='data/postcodes.csv',
    outages_fp: str='docs/data/outages.csv',
    null_threshold: int=1,
    js_template_fp: str='templates/map_js.j2',
    js_docs_fp: str='docs/map.js',
    md_template_fp: str='templates/index_md.j2',
    docs_md_fp: str='docs/index.md'
):
    # Retrieving and parsing outage data
    cleaned_incidents_info = collation.collate_cleaned_incidents_info(dnos=dnos)
    df_outages = collation.get_combined_outages_df(cleaned_incidents_info)
    df_outages = collation.add_outage_longitudes_latitudes(df_outages, postcodes_fp=postcodes_fp, null_threshold=null_threshold)

    # Map generation
    gdf_outages = collation.construct_outages_gdf(df_outages)
    collation.generate_map_js(gdf_outages, js_template_fp=js_template_fp, js_docs_fp=js_docs_fp)

    # Documentation generation
    df_outages_clean = collation.clean_outages_df(df_outages)
    collation.generate_index_md(df_outages_clean, md_template_fp=md_template_fp, docs_md_fp=docs_md_fp)
    df_outages_clean.assign(Incident=df_outages['incident_id']).to_csv(outages_fp, index=False)

    return

# Cell
if __name__ == '__main__' and '__file__' in globals():
    app()


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/01-retrieval.ipynb (unless otherwise specified).

__all__ = ['extract_ukpn_single_incident_ids', 'extract_ukpn_multiple_incident_urls',
           'extract_ukpn_multiple_incident_ids', 'get_ukpn_incident_detail_url', 'is_ukpn_incident_active',
           'extract_ukpn_relevant_info', 'get_ukpn_incidents_info', 'save_json_data', 'clean_ssen_incident_info',
           'get_ssen_incidents_info', 'get_wpd_incident_feed', 'wpd_incident_id_to_url', 'clean_wpd_incident_info',
           'get_wpd_incidents_info', 'get_raw_sp_incidents_info', 'clean_sp_incident_info', 'get_sp_incidents_info',
           'get_np_auth', 'get_np_r', 'get_np_raw_incidents_info', 'clean_np_incident_info', 'get_np_incidents_info',
           'get_enw_incidents_page', 'check_num_results', 'get_enw_raw_incidents', 'clean_enw_incident_info',
           'get_enw_incidents_info']

# Cell
import json
import pandas as pd

import html
import requests
from bs4 import BeautifulSoup as bs
from warnings import warn

from ipypb import track

# Cell
extract_ukpn_single_incident_ids = lambda r_json: [
    incident['PanelContentUrl'].split('incidentId=')[1]
    for incident
    in r_json['Incidents']
    if incident['PowerCutType'] != 'Multiple'
]

# Cell
extract_ukpn_multiple_incident_urls = lambda r_json: [
    f"https://www.ukpowernetworks.co.uk{incident['PanelContentUrl']}"
    for incident
    in r_json['Incidents']
    if incident['PowerCutType'] == 'Multiple'
]

# Cell
def extract_ukpn_multiple_incident_ids(multiple_incident_urls):
    incident_ids = []

    for multiple_incident_url in track(multiple_incident_urls, label='Multiple Ids'):
        r = requests.get(multiple_incident_url)
        soup = bs(r.text, features='lxml')

        incident_ids += [
            link['data-url'].split('incidentId=')[1]
            for link
            in soup.find('div', {'class': 'multiple-incidents--wrapper mb-4'}).findAll('a')
        ]

    return incident_ids

# Cell
get_ukpn_incident_detail_url = lambda incident_id: f'https://www.ukpowernetworks.co.uk/Incidents/getincidentdetails?incidentid={incident_id}'

# Cell
def is_ukpn_incident_active(r_json):
    if r_json['PowerCutType'] == 'Restored':
        return False
    else:
        return True

def extract_ukpn_relevant_info(r_json):
    incident_info = {
        'incident_active': is_ukpn_incident_active(r_json),
        'restored_time': r_json['UKPNIncident']['RestoredDateTime'],
        'estimated_restored_time': r_json['UKPNIncident']['EstimatedRestorationDate'],
        'planned_time': r_json['UKPNIncident']['PlannedDate'],
        'received_time': r_json['UKPNIncident']['ReceivedDate'],
        'postcodes_impacted': r_json['FullPostcodeData'],
        'description': r_json['IncidentCategoryCustomerFriendlyDescription'],
        'incident_url': f"https://www.ukpowernetworks.co.uk/power-cut/map?incidentid={r_json['IncidentReference']}"
    }

    return incident_info

# Cell
def get_ukpn_incidents_info(incidents_url='https://www.ukpowernetworks.co.uk/Incidents/GetIncidents'):
    r_json = requests.get(incidents_url).json()

    incident_ids = (
        extract_ukpn_single_incident_ids(r_json) +
        extract_ukpn_multiple_incident_ids(extract_ukpn_multiple_incident_urls(r_json))
    )

    raw_incidents_info = dict()
    cleaned_incidents_info = dict()

    for incident_id in track(incident_ids, label='Details'):
        try:
            incident_detail_url = get_ukpn_incident_detail_url(incident_id)
            r_json = requests.get(incident_detail_url).json()

            raw_incidents_info[incident_id] = r_json
            cleaned_incidents_info[incident_id] = extract_ukpn_relevant_info(r_json)
        except:
            warn(f'Failed to retrieve incident details for: {incident_id}')

    return raw_incidents_info, cleaned_incidents_info

# Cell
def save_json_data(data, filename, data_dir='../data/raw'):
    with open(f'{data_dir}/{filename}.json', 'w') as fp:
        json.dump(data, fp)

# Cell
def clean_ssen_incident_info(incident):
    ssen_relevant_dates_name_mapping = {
        'LoggedAtUtc': 'received_time',
        'EstimatedArrivalOnSiteUtc': 'estimated_arrival_time',
        'EstimatedRestorationTimeUtc': 'estimated_restored_time'
     }

    cleaned_incident_info = dict()

    for old_dt_name, new_dt_name in ssen_relevant_dates_name_mapping.items():
        dt_str = incident[old_dt_name].split('(')[1].split(')')[0]

        if dt_str != '-62135596800000':
            try:
                dt = pd.to_datetime(int(dt_str)*1e6).strftime('%Y-%m-%d %H:%M:%S')
            except:
                warn(f'{dt_str} could not be parsed to a date')
                dt = None
        else:
            dt = None

        cleaned_incident_info[new_dt_name] = dt

    cleaned_incident_info['postcodes_impacted'] = incident['AffectedAreas']
    cleaned_incident_info['description'] = incident['Message']
    cleaned_incident_info['incident_active'] = True
    cleaned_incident_info['incident_url'] = 'https://www.ssen.co.uk/Powertrack/'

    return cleaned_incident_info

# Cell
def get_ssen_incidents_info(incidents_url='https://www.ssen.co.uk/Sse_Components/Views/Controls/FormControls/PowerTrackHandler.ashx'):
    raw_incidents_info = requests.get(incidents_url, verify=False).json()
    cleaned_incidents_info = dict()

    for incident in track(raw_incidents_info['Faults']):
        incident_ref = incident['Reference']
        cleaned_incidents_info[incident_ref] = clean_ssen_incident_info(incident)

    return raw_incidents_info, cleaned_incidents_info

# Cell
def get_wpd_incident_feed(url='https://powercuts.westernpower.co.uk'):
    r = requests.get(url)

    raw_incidents_info = json.loads(html.unescape(r.text.split('data-ng-init="init(')[1].split(')"></div><div id="powercuts"')[0]))
    feed = json.loads(raw_incidents_info['feed'])

    return feed

# Cell
wpd_incident_id_to_url = lambda incident_id: f'https://powercuts.westernpower.co.uk/__powercuts/getIncidentById?incidentId={incident_id}'

# Cell
def clean_wpd_incident_info(incident):
    wpd_relevant_dates_name_mapping = {
        'startTime': 'received_time',
        'etr': 'estimated_restored_time'
     }

    cleaned_incident_info = dict()

    for old_dt_name, new_dt_name in wpd_relevant_dates_name_mapping.items():
        if incident[old_dt_name] is not None:
            cleaned_incident_info[new_dt_name] = pd.to_datetime(incident[old_dt_name]*1e6).strftime('%Y-%m-%d %H:%M:%S')
        else:
            cleaned_incident_info[new_dt_name] = None

    cleaned_incident_info['postcodes_impacted'] = incident['postcodes']
    cleaned_incident_info['description'] = incident['status']
    cleaned_incident_info['incident_active'] = bool(1 - incident['restored'])
    cleaned_incident_info['incident_url'] = f"https://powercuts.westernpower.co.uk/incident/{incident['id']}"

    return cleaned_incident_info

# Cell
def get_wpd_incidents_info(incidents_url='https://powercuts.westernpower.co.uk'):
    cleaned_incidents_info = dict()

    raw_incidents_info = get_wpd_incident_feed(incidents_url)
    incident_ids = [incident['id'] for incident in raw_incidents_info['incidents']]

    for incident_id in incident_ids:
        incident_url = wpd_incident_id_to_url(incident_id)
        r_json = requests.get(incident_url).json()
        cleaned_incidents_info[incident_id] = clean_wpd_incident_info(r_json)

    return raw_incidents_info, cleaned_incidents_info

# Cell
get_raw_sp_incidents_info = lambda sp_map_url='https://www.spenergynetworks.co.uk/pages/power_cuts_map.aspx': json.loads(requests.get(sp_map_url).text.split('arrPowercutsPostcodes:')[1].split('strPagePathListView')[0].replace(',\r\n', '').strip())

# Cell
def clean_sp_incident_info(incident_info):
    def clean_dt(dt):
        if dt is None:
            return ''
        else:
            return pd.to_datetime(dt, utc=True).strftime('%Y-%m-%d %H:%M')

    cleaned_incident_info = dict()

    cleaned_incident_info['received_time'] = clean_dt(incident_info['CREATION_DATE'])
    cleaned_incident_info['estimated_restored_time'] = clean_dt(incident_info['EST_REST_DATE'])
    cleaned_incident_info['postcodes_impacted'] = incident_info['POSTCODES']
    cleaned_incident_info['description'] = incident_info['MAIN_MESSAGE']
    cleaned_incident_info['incident_active'] = incident_info['HISTORIC_FLAG'] is None
    cleaned_incident_info['incident_url'] = f"https://www.spenergynetworks.co.uk/pages/power_cuts_map.aspx?incRef={incident_info['INCIDENT_REF']}"

    return cleaned_incident_info

# Cell
def get_sp_incidents_info(sp_map_url='https://www.spenergynetworks.co.uk/pages/power_cuts_map.aspx'):
    cleaned_incidents_info = dict()

    raw_incidents_info = get_raw_sp_incidents_info(sp_map_url)

    for incident_info in raw_incidents_info:
        incident_id = incident_info['INCIDENT_REF']
        cleaned_incidents_info[incident_id] = clean_sp_incident_info(incident_info)

    return raw_incidents_info, cleaned_incidents_info

# Cell
def get_np_auth(np_main_url='https://www.northernpowergrid.com/power-cuts'):
    s = requests.Session()

    r = s.get(np_main_url)
    soup = bs(r.text, features='lxml')

    authenticity_token = soup.find('input', attrs={'name': 'authenticityToken'})['value']

    return s, authenticity_token

# Cell
def get_np_r(
    np_main_url='https://www.northernpowergrid.com/power-cuts',
    np_incidents_url='https://www.northernpowergrid.com/powercutsgetallbyincno'
):
    s, authenticity_token = get_np_auth(np_main_url)

    data = {
        'method': 'incno',
        'categoryFilters': 'Service Cutout Change,Asset repairs by Troublecall,Metering,Emergency Disconnection,Emergency Disconnection (Charge),Cat A,Cat B,Cat C',
        'authenticityToken': authenticity_token
    }

    r = s.post(np_incidents_url, data=data)

    return r

# Cell
get_np_raw_incidents_info = lambda r: json.loads(r.json()['data'])

# Cell
def clean_np_incident_info(incident_info):
    def clean_dt(dt):
        if dt is None:
            return ''
        else:
            return pd.to_datetime(dt, unit='ms').strftime('%Y-%m-%d %H:%M')

    cleaned_incident_info = dict()

    cleaned_incident_info['received_time'] = clean_dt(incident_info['logged'])
    cleaned_incident_info['estimated_restored_time'] = clean_dt(incident_info['estimatedTimeTillResolution'])
    cleaned_incident_info['postcodes_impacted'] = incident_info['postcodes']
    cleaned_incident_info['description'] = f"{incident_info['category']} {incident_info['status']}"
    cleaned_incident_info['incident_active'] = (incident_info['totalConfirmedOff'] + incident_info['totalPredictedOff']) > 0
    cleaned_incident_info['incident_url'] = 'https://www.northernpowergrid.com/power-cuts'

    return cleaned_incident_info

# Cell
def get_np_incidents_info(
    np_main_url='https://www.northernpowergrid.com/power-cuts',
    np_incidents_url='https://www.northernpowergrid.com/powercutsgetallbyincno'
):
    cleaned_incidents_info = dict()

    r = get_np_r(np_main_url, np_incidents_url)
    raw_incidents_info = get_np_raw_incidents_info(r)

    for incident_id, incident_info in raw_incidents_info['powercuts'].items():
        cleaned_incidents_info[incident_id] = clean_np_incident_info(incident_info)

    return raw_incidents_info, cleaned_incidents_info

# Cell
get_enw_incidents_page = lambda page=1, page_size=10000: f'https://www.enwl.co.uk/power-outages/search?pageSize={page_size}&postcodeOrReferenceNumber=&pageNumber={page}&includeCurrent=true&includeResolved=true&includeTodaysPlanned=true&includeFuturePlanned=true&includeCancelledPlanned=true'

# Cell
def check_num_results(r_json):
    num_total_results = r_json['TotalResults']
    num_results_returned = len(r_json['Items'])

    if num_total_results != num_results_returned:
        warn(f'Only {num_results_returned} items were returned for Electricity North West when there are {num_total_results} in total')

    return

# Cell
def get_enw_raw_incidents(page=1, page_size=10000):
    url = get_enw_incidents_page(page=page, page_size=page_size)
    raw_incidents = requests.get(url).json()
    check_num_results(raw_incidents)

    return raw_incidents

# Cell
def clean_enw_incident_info(incident_info):
    def clean_dt(dt):
        if dt is None:
            return ''
        else:
            return pd.to_datetime(dt).strftime('%Y-%m-%d %H:%M')

    cleaned_incident_info = dict()
    cleaned_incident_info['received_time'] = clean_dt(incident_info['date'])
    cleaned_incident_info['estimated_restored_time'] = clean_dt(incident_info['estimatedTimeOfRestorationMajority'])
    cleaned_incident_info['postcodes_impacted'] = incident_info['AffectedPostcodes'].strip().split(', ')
    cleaned_incident_info['description'] = incident_info['AdditionalFaultInfo']
    cleaned_incident_info['incident_active'] = incident_info['FaultLabel'] in ['CurrentFault', 'Live power cut']
    cleaned_incident_info['incident_url'] = 'https://www.enwl.co.uk/power-cuts/power-cuts-power-cuts-live-power-cut-information-fault-list/fault-list'

    return cleaned_incident_info

# Cell
def get_enw_incidents_info(page=1, page_size=10000):
    cleaned_incidents_info = dict()

    raw_incidents_info = get_enw_raw_incidents(page=page, page_size=page_size)

    for incident_info in raw_incidents_info['Items']:
        incident_id = incident_info['faultNumber']
        cleaned_incidents_info[incident_id] = clean_enw_incident_info(incident_info)

    return raw_incidents_info, cleaned_incidents_info


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/02-collation.ipynb (unless otherwise specified).

__all__ = ['collate_cleaned_incidents_info', 'get_combined_outages_df', 'check_postcode_is_in_df',
           'get_postcode_location_df', 'get_postcode_url', 'get_postcode_location_api', 'PostCodeLocator',
           'add_outage_longitudes_latitudes', 'construct_outages_gdf', 'generate_map_js', 'clean_outages_df',
           'generate_index_md']

# Cell
import json
import numpy as np
import pandas as pd
import geopandas as gpd

from outage import retrieval

import os
import requests
from warnings import warn
from jinja2 import Template

from ipypb import track

# Cell
def collate_cleaned_incidents_info(dnos=['ukpn', 'ssen', 'wpd', 'sp', 'np', 'enw']):
    cleaned_incidents_info = dict()

    for dno in track(dnos, label='DNOs'):
        raw_dno_incidents_info, cleaned_dno_incidents_info = getattr(retrieval, f'get_{dno}_incidents_info')()
        cleaned_incidents_info[dno] = cleaned_dno_incidents_info
        retrieval.save_json_data(raw_dno_incidents_info, f'{dno}_incidents_info')

    return cleaned_incidents_info

# Cell
def get_combined_outages_df(cleaned_incidents_info):
    df_outages = pd.DataFrame()

    for dno, dno_outages in cleaned_incidents_info.items():
        df_dno_outages = pd.DataFrame(dno_outages).T.query('incident_active==True')

        df_dno_outages['dno'] = dno
        df_dno_outages = df_dno_outages[['received_time', 'estimated_restored_time', 'postcodes_impacted', 'description', 'dno', 'incident_url']]

        df_dno_outages.index.name = 'incident_id'
        df_dno_outages = df_dno_outages.reset_index()

        df_outages = df_outages.append(df_dno_outages)

    df_outages = df_outages.reset_index(drop=True)

    return df_outages

# Cell
check_postcode_is_in_df = lambda postcode, df: postcode in df.index

# Cell
def get_postcode_location_df(df_postcodes, postcode):
    postcode_is_in_df = check_postcode_is_in_df(postcode, df_postcodes)

    if postcode_is_in_df:
        location = df_postcodes.loc[postcode]

        longitude = location['longitude']
        latitude = location['latitude']

    else:
        longitude, latitude = None, None

    return longitude, latitude

# Cell
get_postcode_url = lambda postcode: f'http://api.getthedata.com/postcode/{postcode}'

# Cell
def get_postcode_location_api(postcode):
    longitude, latitude = None, None

    postcode_url = get_postcode_url(postcode)
    r_json = requests.get(postcode_url).json()

    if r_json['status'] == 'match':
        data_keys = r_json['data'].keys()

        if 'longitude' in data_keys and 'latitude' in data_keys:
            longitude = float(r_json['data']['longitude'])
            latitude = float(r_json['data']['latitude'])

    return longitude, latitude

# Cell
class PostCodeLocator:
    def __init__(self, postcodes_fp: str='../data/postcodes.csv'):
        # Loading Post Codes DataFrame
        df_postcodes = pd.read_csv(postcodes_fp, index_col='postcode')
        df_postcodes.index = df_postcodes.index.str.replace(' ', '')

        self.df_postcodes = df_postcodes

    def get_postcode_location(self, postcode):
        postcode_is_in_df = check_postcode_is_in_df(postcode, self.df_postcodes)

        if postcode_is_in_df:
            longitude, latitude = get_postcode_location_df(self.df_postcodes, postcode)

        else:
            longitude, latitude = get_postcode_location_api(postcode)

        if longitude is None or latitude is None:
            warn(f'The longitude and latitude of {postcode} could not be determined')

        return longitude, latitude

# Cell
def add_outage_longitudes_latitudes(df_outages, postcodes_fp: str='../data/postcodes.csv', null_threshold=1):
    pcl = PostCodeLocator(postcodes_fp)

    s_incident_longitudes = pd.Series(index=df_outages.index, dtype='float64')
    s_incident_latitudes = pd.Series(index=df_outages.index, dtype='float64')

    for outage_idx, s_outage in track(df_outages.iterrows(), total=df_outages.shape[0]):
        incident_longitudes = []
        incident_latitudes = []

        for postcode_impacted in s_outage['postcodes_impacted']:
            longitude, latitude = pcl.get_postcode_location(postcode_impacted)

            incident_longitudes += [longitude]
            incident_latitudes += [latitude]

        s_incident_longitudes.loc[outage_idx] = pd.Series(incident_longitudes).mean()
        s_incident_latitudes.loc[outage_idx] = pd.Series(incident_latitudes).mean()

    df_outages = (df_outages
                  .assign(longitude=s_incident_longitudes)
                  .assign(latitude=s_incident_latitudes)
                 )

    idxs_to_keep = (df_outages.isnull().sum(axis=1)<=null_threshold).replace(False, np.nan).dropna().index
    df_outages = df_outages.loc[idxs_to_keep]

    return df_outages

# Cell
construct_outages_gdf = lambda df_outages: gpd.GeoDataFrame(
    data=df_outages.drop(columns=['longitude', 'latitude']),
    geometry=gpd.points_from_xy(df_outages['longitude'], df_outages['latitude']),
    crs='EPSG:4326'
)

# Cell
def generate_map_js(gdf_outages, js_template_fp='templates/map_js.j2', js_docs_fp='docs/map.js'):
    outages_geojson = json.loads(gdf_outages.to_json(drop_id=True))

    rendered_map_js = Template(open(js_template_fp).read()).render(
        zoom=5,
        center=[53.96, -3.22],
        geojson_features=str(outages_geojson).replace('None', 'null')
    )

    with open(js_docs_fp, 'w', encoding='utf8') as fp:
        fp.write(rendered_map_js)

    return

# Cell
def clean_outages_df(df_outages):
    df_outages_clean = pd.DataFrame()

    df_outages_clean['Incident'] = df_outages.agg(lambda x: f"<a href=\"{x['incident_url']}\" target=\"_blank\">{x['incident_id']}</a>", axis=1)
    df_outages_clean['Received Time'] = pd.to_datetime(df_outages['received_time']).dt.strftime('%Y-%m-%d %H:%M').fillna('Unknown')
    df_outages_clean['Estimated Restored Time'] = pd.to_datetime(df_outages['estimated_restored_time']).dt.strftime('%Y-%m-%d %H:%M').fillna('Unknown')
    df_outages_clean['Postcodes Impacted'] = df_outages['postcodes_impacted'].str.join(', ')
    df_outages_clean['Operator'] = df_outages['dno'].str.upper()
    df_outages_clean['Description'] = df_outages['description'].str.replace('\n', '<br>')

    df_outages_clean = df_outages_clean.sort_values('Received Time', ascending=False)

    return df_outages_clean

# Cell
def generate_index_md(df_outages_clean, md_template_fp='templates/index_md.j2', docs_md_fp='docs/index.md'):
    rendered_index_md = Template(open(md_template_fp).read()).render(
        map_width='600px',
        map_height='500px',
        md_table=df_outages_clean.to_markdown(index=False)
    )

    with open(docs_md_fp, 'w', encoding='utf8') as fp:
        fp.write(rendered_index_md)

    return


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"extract_ukpn_single_incident_ids": "01-retrieval.ipynb",
         "extract_ukpn_multiple_incident_urls": "01-retrieval.ipynb",
         "extract_ukpn_multiple_incident_ids": "01-retrieval.ipynb",
         "get_ukpn_incident_detail_url": "01-retrieval.ipynb",
         "is_ukpn_incident_active": "01-retrieval.ipynb",
         "extract_ukpn_relevant_info": "01-retrieval.ipynb",
         "get_ukpn_incidents_info": "01-retrieval.ipynb",
         "save_json_data": "01-retrieval.ipynb",
         "clean_ssen_incident_info": "01-retrieval.ipynb",
         "get_ssen_incidents_info": "01-retrieval.ipynb",
         "get_wpd_incident_feed": "01-retrieval.ipynb",
         "wpd_incident_id_to_url": "01-retrieval.ipynb",
         "clean_wpd_incident_info": "01-retrieval.ipynb",
         "get_wpd_incidents_info": "01-retrieval.ipynb",
         "get_raw_sp_incidents_info": "01-retrieval.ipynb",
         "clean_sp_incident_info": "01-retrieval.ipynb",
         "get_sp_incidents_info": "01-retrieval.ipynb",
         "get_np_auth": "01-retrieval.ipynb",
         "get_np_r": "01-retrieval.ipynb",
         "get_np_raw_incidents_info": "01-retrieval.ipynb",
         "clean_np_incident_info": "01-retrieval.ipynb",
         "get_np_incidents_info": "01-retrieval.ipynb",
         "get_enw_incidents_page": "01-retrieval.ipynb",
         "check_num_results": "01-retrieval.ipynb",
         "get_enw_raw_incidents": "01-retrieval.ipynb",
         "clean_enw_incident_info": "01-retrieval.ipynb",
         "get_enw_incidents_info": "01-retrieval.ipynb",
         "collate_cleaned_incidents_info": "02-collation.ipynb",
         "get_combined_outages_df": "02-collation.ipynb",
         "check_postcode_is_in_df": "02-collation.ipynb",
         "get_postcode_location_df": "02-collation.ipynb",
         "get_postcode_url": "02-collation.ipynb",
         "get_postcode_location_api": "02-collation.ipynb",
         "PostCodeLocator": "02-collation.ipynb",
         "add_outage_longitudes_latitudes": "02-collation.ipynb",
         "construct_outages_gdf": "02-collation.ipynb",
         "generate_map_js": "02-collation.ipynb",
         "clean_outages_df": "02-collation.ipynb",
         "generate_index_md": "02-collation.ipynb",
         "app": "03-deployment.ipynb",
         "update_docs": "03-deployment.ipynb"}

modules = ["retrieval.py",
           "collation.py",
           "deployment.py"]

doc_url = "https://AyrtonB.github.io/outage/"

git_url = "https://github.com/AyrtonB/Outage-Watch"

def custom_doc_links(name): return None
